#ifndef _STIM_UTIL_TOP_MBQC_DECOMPOSITION_H
#define _STIM_UTIL_TOP_MBQC_DECOMPOSITION_H

#include "stim/gates/gates.h"

namespace stim {

const char *mbqc_decomposition(GateType gate);

}  // namespace stim

#endif
